//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
//   Important note about DLL memory management when your DLL uses the
//   static version of the RunTime Library:
//
//   If your DLL exports any functions that pass String objects (or structs/
//   classes containing nested Strings) as parameter or function results,
//   you will need to add the library MEMMGR.LIB to both the DLL project and
//   any other projects that use the DLL.  You will also need to use MEMMGR.LIB
//   if any other projects which use the DLL will be perfomring new or delete
//   operations on any non-TObject-derived classes which are exported from the
//   DLL. Adding MEMMGR.LIB to your project will change the DLL and its calling
//   EXE's to use the BORLNDMM.DLL as their memory manager.  In these cases,
//   the file BORLNDMM.DLL should be deployed along with your DLL.
//
//   To avoid using BORLNDMM.DLL, pass string information using "char *" or
//   ShortString parameters.
//
//   If your DLL uses the dynamic version of the RTL, you do not need to
//   explicitly add MEMMGR.LIB as this will be done implicitly for you
//---------------------------------------------------------------------------
// USERES("invertFunc.res");
//---------------------------------------------------------------------------
#include "../danImage.h"
#include <math.h>

#define MIN( a, b ) ((a)<(b) ? (a) : (b))

#define currPixelX 10
#define currPixelY 10

//---------------------------------------------------------------------------

extern "C" __declspec(dllexport) char *getName();
extern "C" __declspec(dllexport) char *getInfo();
extern "C" __declspec(dllexport) danImage *filterFunction(danImage *src);

//Function prototype for Catmull-Rom

float RiCatmullRomFilter(float, float, float, float);

// Specify Name & Description (no more than 20 characters!!
char name[21] = "Catmull-Rom Filter";
char desc[21] = "Complex Slow Filter";

//---------------------------------------------------------------------------
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void*)
{
        return 1;
}
//---------------------------------------------------------------------------
char *getName()
{
        return name;
}

char *getInfo()
{
        return desc;
}

 // see project report for details on this looping function

danImage *filterFunction(danImage *src)
{
    danImage *temp;
    double d;
    double x,y,i,j;

    double filterWidth = 3.;   //pixelwidth -1
    double filterHeight = 3.;  //pixelwidth -1

    double sampleIncrement = 1;

    temp = new danImage;

    temp->setWidth(src->getWidth());
    temp->setHeight(src->getHeight());

    // NOTE:
    // i : X Position in Dest Image
    // j : Y Position in Dest Image
    // x : X Position in Filter (Sample Space)
    // y : Y Position in Filter (Sample Space)




for (i=0; i<src->getWidth(); i++)
for (j=0; j<src->getHeight(); j++)
{
        double totalR = 0;
        double totalG = 0;
        double totalB = 0;
        double noPix = 0;

    for (x = -(filterWidth/2); x <= filterWidth/2; x+=sampleIncrement)
    for (y = -(filterHeight/2); y <= filterHeight/2; y+=sampleIncrement)
    {

        if (((floor(i+x)>=0) && (floor(j+y)>=0)) && ((floor(i+x)<src->getWidth()) && (floor(j+y)<src->getHeight())))
        {

            d = 0;

          d = RiCatmullRomFilter(x,y,filterWidth, filterHeight);

        if (d>0)
        {
                totalR += d*((double)src->getPixelRed( (floor(i+x)),(floor(j+y)) ) );
                totalG += d*((double)src->getPixelGreen( (floor(i+x)),(floor(j+y)) ) );
                totalB += d*((double)src->getPixelBlue( (floor(i+x)),(floor(j+y)) ) );
                noPix +=d;
        }

            // Origin in sample space corresponds to the current pixel
            // being looked at.
            // E.g If we are working on Pixel[10,10]
            //     The middle of pixel[10,10] would correspond
            //	   to 0,0 in sample space.

        }
   }

// SET PIXEL WITH AVERAGE
    temp->setPixel(i,j,
                        (int)(totalR/noPix),
                        (int)(totalG/noPix),
                        (int)(totalB/noPix)
                                );

//         temp->setPixel(i,j,src->getPixelRed(i,j),src->getPixelRed(i,j),src->getPixelGreen(i,j));

        // result = total / noPix;


}

    return temp;
}
//---------------------------------------------------------------------------

//Function Definition for CatmullRom

float RiCatmullRomFilter(float x, float y, float xwidth, float ywidth)
{
	double d, d1, d2, d3;

    d = sqrt(x*x+y*y);
    d1 = abs(d);
    d2 = d*d;
    d3 = d2*d1;

    if (d<1)
    	return ( (3/2)*d3 - (5/2)*d2 - 1);
    else if (d1 < 2)
    	return ( (-0.5)*d3 + (5/2)*d2 - 4*d1 + 2);
    else
    	return 0.0;
}

